/*
Final Frontier Trader
by TomT64

Requires the following libraries:
SDL >= 1.2.4
SDL_image >= 1.2.2
SDL_mixer >= 1.2.4
SDL_ttf >= 2.0.6
SDL_gfx >= 2.0.8
physfs (with zip support) >= 0.1.9

SDL and its libraries may be found easily through Google
or SDL's website (http://www.libsdl.org).
PhysFS may be found at its website (http://www.icculus.org/physfs/)
*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "SDL.h"
#include "SDL_syswm.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"
#include "SDL_framerate.h"

#include "startup.h"
#include "fftfile.h"
#include "input.h"
#include "graphics.h"
#include "gui.h"
#include "guiapply.h"
#include "units.h"
#include "scenario.h"

Uint8* keys;
int SCREEN_WIDTH;
int SCREEN_HEIGHT;
TTF_Font *font;
TTF_Font *font2;

FFTInput *fftinput;
Preferences *prefs;
StarField *stars;
StarField *stars2;
Scenario *mainScen;
Unit *ship;
Map *map;
GUI *gui2;
Mix_Chunk *shot;
Mix_Chunk *explode;

int SMOOTH=1;
bool FPSSHOW=false;
bool LOG=false;
std::string VERSION="0.65";

static int mainLoop();
static int AskToQuitGame();
static int InGameMenu();
static void LoadScenario();

#define PI 3.1415926535897932384626433832795

SDL_Color white = { 0xFF, 0xFF, 0xFF, 0 };
SDL_Color yellow = { 0xFF, 0xFF, 0x00, 0 };
SDL_Color black = { 0x00, 0x00, 0x00, 0 };

int main ( int argc, char *argv[] )
{
	bool fullscreen=false;
	bool software=false;
	if (argc>1)
	{
		int i;
		for (i=1; i<argc; i++)
		{
			if (!strcmp(argv[i],"-nosmooth"))
			{
				SMOOTH=0;
			}
			if (!strcmp(argv[i],"-fullscreen"))
			{
				fullscreen=true;
			}
			if (!strcmp(argv[i],"-software"))
			{
				software=true;
			}
			if (!strcmp(argv[i],"-showfps"))
			{
				FPSSHOW=true;
			}
			if (!strcmp(argv[i],"-log"))
			{
				LOG=true;
			}
		}
	}
	if (LOG) printf("Final Frontier Trader\nversion %s\nby TomT64\n------------\n\n",VERSION.c_str());
	// We initalize the random number generator
	srand(time(NULL));

	// We store the state of keys here

	SDL_Surface *screen=SDL_GetVideoSurface();
	fftinput=new FFTInput;
	int init=PHYSFS_init(argv[0]); //init physfs
	if (!init)
	{
		printf("Failed to initialize PHYSFS.  Reason: %s\n",PHYSFS_getLastError());
		exit(1);
	}
	if (LOG)
	{
		printf("PHYSFS initialized successfully\n");
		char **i;
		for (i = PHYSFS_getSearchPath(); *i != NULL; i++)
		printf("[%s] is in the search path.at the beginning of execution\n", *i);
	}
	PHYSFS_addToSearchPath(PHYSFS_getBaseDir(), 0); //fpk dir
	PHYSFS_setWriteDir(PHYSFS_getBaseDir());
	if (LOG) printf("Loading Preferences\n");
	prefs=new Preferences("prefs");
	delete fftinput;
	fftinput=prefs->GetInput();
	PHYSFS_addToSearchPath("fpk/base.fpk", 1); //load base packfile
	SCREEN_WIDTH = 640;
	SCREEN_HEIGHT = 480;

	if(TTF_Init()==-1) {
	printf("TTF_Init: %s\n", TTF_GetError());
	exit(2);
	}
	if (LOG) printf("SDL_ttf initialized successfully\n");

	// load font
	font=LoadTTF("fonts/arial.ttf", 10);
	if(!font) {
	printf("Unable to Open font 'fonts/arial.ttf' -  TTF_OpenFont: %s\n", TTF_GetError());
	// handle error
	}
	font2=LoadTTF("fonts/arial.ttf", 14);
	if(!font2) {
	printf("Unable to Open font 'fonts/arial.ttf' - TTF_OpenFont: %s\n", TTF_GetError());
	// handle error
	}
	if (LOG) printf("Main Fonts loaded\n");

	/* initialize SDL */
	#ifdef DEBUG
	if( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_NOPARACHUTE) < 0 ) {
	fprintf(stderr, "Could not initialize SDL: %s\n",
			SDL_GetError());
		return -1;
	}
	#else
	if( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0 ) {
	fprintf(stderr, "Could not initialize SDL: %s\n",
			SDL_GetError());
		return -1;
	}
	#endif

	if (LOG) printf("SDL initialized successfully\n");

	//atexit(SDL_Quit);

	/* set the title bar */
	SDL_WM_SetCaption("Final Frontier Trader", "Final Frontier Trader");
	SDL_Surface *icon = LoadICON("gfx/ico/fftico.png");
	SDL_WM_SetIcon(icon, NULL);
	if (LOG) printf("Icon and caption in place\n");

	/* create window */
	screen = SDL_SetVideoMode(400, 300, 0, SDL_HWSURFACE|SDL_DOUBLEBUF);
	//CenterWindow();
	if (LOG) printf("Video mode set and centered\n");

	SDL_Surface *bg = LoadIMG("gfx/startupbg.png");
	DrawIMG(bg,0,0);
	SDL_Flip(screen);
	if (LOG) printf("Splash screen displayed\n");

	stars = new StarField;
	stars2 = new StarField;
	stars->Init(SCREEN_WIDTH, SCREEN_HEIGHT);
	stars2->Init(SCREEN_WIDTH, SCREEN_HEIGHT);
	if (LOG) printf("Beginning warp stars initialized\n");

	char **k;
	std::string fpktemp=" ";
	char **fpklist=PHYSFS_enumerateFiles("fpk");
	for (k = fpklist; *k != NULL; k++)
	{
		if (strcmp(fpktemp.c_str(),"base.fpk") && IsFPK(*k))
		{
			fpktemp="fpk/";
			fpktemp+=*k;
			PHYSFS_addToSearchPath(fpktemp.c_str(), 1);
		}
	}
	PHYSFS_freeList(fpklist);
	if (LOG) printf("All FPKs placed in search path\n");

	ship = new Unit;
	ship->Init(0, 0, "gfx/ship/ship.png","base.fpk");
	if (LOG) printf("Main ship initialized\n");

	map = new Map;
	map->Init();
	if (LOG) printf("Map initialized\n");

	if(Mix_OpenAudio(22050, AUDIO_S16SYS, 2, 2048) < 0)
	{
		printf("Warning: Couldn't set 22050 Hz 16-bit audio\n\
		- Reason: %s\n", SDL_GetError());
	}
	shot = LoadSound("sound/shoot.wav");
	explode = LoadSound("sound/expl01.wav");
	if (LOG) printf("Audio initialized and sounds loaded\n");

	gui2=new GUI;
	gui2->Init();

	SDL_Delay(1000);

	//reset res
	if (fullscreen && software)
	{
		if (SDL_VideoModeOK(SCREEN_WIDTH, SCREEN_HEIGHT, 32, SDL_SWSURFACE|SDL_FULLSCREEN))
		{
			screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 32, SDL_SWSURFACE|SDL_FULLSCREEN);
		}
		else exit(0);
	}
	else if (fullscreen)
	{
		if (SDL_VideoModeOK(SCREEN_WIDTH, SCREEN_HEIGHT, 32, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN))
		{
			screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 32, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
		}
		else exit(0);
	}
	else if (software) screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_SWSURFACE);
	else screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_HWSURFACE|SDL_DOUBLEBUF);
	CenterWindow();
	if (LOG)
	{
		printf("Main window set to 640x480 ");
		if (fullscreen) printf("at fullscreen ");
		else printf("at windowed ");
		if (software) printf("software mode");
		else printf("hardware mode");
		printf(" and centered\n");
	}

	int done=0;
	int maintemp;
	int GUIresult;
	if (LOG) printf("Proceeding to main loop\n");
	while(!done)
	{
		GUIresult=RunStartingGUI(SCREEN_WIDTH, SCREEN_HEIGHT);
		if (LOG) printf("Option from Starting GUI selected\n");
		if (GUIresult==-1)
		{
			done=1;
			if (LOG) printf("Exiting game\n");
		}
		if (GUIresult==2)
		{
			if (LOG) printf("Entering New Game\n");
			LoadScenario();
			if (LOG) printf("Proceeding to main game play\n");
			maintemp=mainLoop();
			if (LOG) printf("Main gameplay exited\n");
			if (maintemp==-1) done=1;
		}
	}

	/* clean up */
	SDL_FreeSurface(bg);
	delete fftinput;
	delete gui2;
	delete stars;
	delete stars2;
	delete ship;
	delete map;
	delete mainScen;
	Mix_FreeChunk(shot);
	Mix_FreeChunk(explode);
	Mix_CloseAudio();
	SDL_FreeSurface(icon);
	TTF_CloseFont(font);
	TTF_CloseFont(font2);
	PHYSFS_deinit();
	TTF_Quit();
	SDL_Quit();
	if (LOG) printf("Thank you for playing!\n");

	return 0;
}

static int AskToQuitGame()
{
	if (LOG) printf("AskToQuitGame()\n");
	SDL_Event event;
	SDL_Surface *screen=SDL_GetVideoSurface();
	SDL_Surface *bg = SDL_CreateRGBSurface(0, screen->w, screen->h, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
	SDL_Rect dest;
	dest.x = 0;
	dest.y = 0;
	SDL_BlitSurface(screen, NULL, bg, &dest);
	GUI *gui3=new GUI;
	gui3->Init();
	gui3->MakeWindow(120,200,SCREEN_WIDTH-240,110);
	gui3->MakeStatic(30,206,SCREEN_WIDTH-60,20,"Are you sure you want to quit this game?");
	gui3->MakeButton(150,270,40,"Yes");
	gui3->MakeButton(SCREEN_WIDTH-190,270,40,"No");
	int yesButton=gui3->GetButtonID(150,270,40,"Yes");
	int noButton=gui3->GetButtonID(SCREEN_WIDTH-190,270,40,"No");
	int done=0;
	int getx, gety, click;
	while(!done)
	{
		Slock();
		while (SDL_PollEvent(&event))
		{
			// an event was found
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				delete fftinput;
				delete gui2;
				delete gui3;
				delete stars;
				delete stars2;
				delete ship;
				delete map;
				Mix_FreeChunk(shot);
				Mix_FreeChunk(explode);
				Mix_CloseAudio();
				TTF_CloseFont(font);
				TTF_CloseFont(font2);
				PHYSFS_deinit();
				TTF_Quit();
				SDL_Quit();
				exit(0);
				break;

			// handle the keyboard
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_RETURN:
					done = 1;
					break;
				default:
					break;
				}
				break;
			case SDL_MOUSEMOTION:
				SDL_GetMouseState(&getx, &gety);
				gui3->SendMouseMotion(getx,gety);
				break;
			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&getx, &gety);
				click = gui3->SendMouseClick(getx, gety);
				break;
			case SDL_MOUSEBUTTONUP:
				SDL_GetMouseState(&getx, &gety);
				if (click!=-1 && click==gui3->SendMouseClick(getx, gety))
				{
					if (click==yesButton)
					{
						SDL_FillRect(screen,0,0);
						DrawIMG(bg,0,0);
						gui3->Draw();
						Sulock();
						SDL_Flip(screen);
						SDL_Delay(100);
						done=1;
					}
					if (click==noButton)
					{
						SDL_FillRect(screen,0,0);
						DrawIMG(bg,0,0);
						gui3->Draw();
						Sulock();
						SDL_Flip(screen);
						SDL_Delay(100);
						done=-1;
					}
				}
				gui3->UpdateButtons();
				break;
			default:
				break;
			}
		}
		SDL_FillRect(screen,0,0);
		DrawIMG(bg,0,0);
		gui3->Draw();
		Sulock();
		SDL_Flip(screen);
	}
	delete gui3;
	SDL_FreeSurface(bg);
	if (done!=1) done=0;
	return done;
}

static int InGameMenu()
{
	if (LOG) printf("InGameMenu()\n");
	SDL_Event event;
	SDL_Surface *screen=SDL_GetVideoSurface();
	int w=screen->w;
	int h=screen->h;
	SDL_Surface *bg = SDL_CreateRGBSurface(0, screen->w, screen->h, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
	SDL_Rect dest;
	dest.x = 0;
	dest.y = 0;
	SDL_BlitSurface(screen, NULL, bg, &dest);
	GUI *optionsGUI=new GUI;
	optionsGUI->Init();
	optionsGUI->MakeWindow(60, 120, w-120, 220);
	optionsGUI->MakeStatic(60,126,w-120,20,"Menu");
	int sControls=optionsGUI->MakeButton(100,160,w-200, "Controls");
	int sCancel=optionsGUI->MakeButton(100,280,200, "Back to Game");
	int sExit=optionsGUI->MakeButton(w-300,280,200, "Exit Game");

	int done=0;
	int getx, gety, click;
	while(!done)
	{
		Slock();
		while (SDL_PollEvent(&event))
		{
			// an event was found
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				done=-1;
				break;

			// handle the keyboard
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_ESCAPE:
					done=AskToQuitGame();
					break;
				default:
					break;
				}
				break;
			case SDL_MOUSEMOTION:
				SDL_GetMouseState(&getx, &gety);
				optionsGUI->SendMouseMotion(getx,gety);
				break;
			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&getx, &gety);
				click = optionsGUI->SendMouseClick(getx, gety);
				break;
			case SDL_MOUSEBUTTONUP:
				SDL_GetMouseState(&getx, &gety);
				if (click!=-1 && click==optionsGUI->SendMouseClick(getx, gety))
				{
					SDL_FillRect(screen,0,0);
					DrawIMG(bg,0,0);
					optionsGUI->Draw();
					Sulock();
					SDL_Flip(screen);
					if (click==sControls)
					{
						done=ControlChanging(w,h,bg,false);
						if (done==1) done=0;
					}
					if (click==sCancel)
					{
						done=2;
					}
					if (click==sExit)
					{
						done=AskToQuitGame();
					}
				}
				optionsGUI->UpdateButtons();
				break;
			default:
				break;
			}
		}
		SDL_FillRect(screen,0,0);
		DrawIMG(bg,0,0);
		optionsGUI->Draw();
		Sulock();
		SDL_Flip(screen);
	}
	delete optionsGUI;
	SDL_FreeSurface(bg);
	if (done!=1 && done!=-1) done=0;
	return done;
}

static void LoadScenario()
{
	if (LOG) printf("LoadScenario()\n");
	SDL_Surface *screen=SDL_GetVideoSurface();
	if (LOG) printf("Entering Scenario selection\n");
	std::string scenselect=ChooseScenario();
	if (LOG) printf("Scenario Selection is %s\n",scenselect.c_str());
	delete mainScen;
	mainScen = new Scenario();
	if (LOG) printf("Reading Scenario information\n");
	mainScen->ReadOnly(scenselect);

	if (LOG) printf("Entering ship selection\n");
	ship->SetZoom(1);
	int quit=NewShipOptions(SCREEN_WIDTH, SCREEN_HEIGHT, mainScen);
	if (quit==-1)
	{
		delete fftinput;
		delete gui2;
		delete stars;
		delete stars2;
		delete ship;
		delete map;
		delete mainScen;
		Mix_FreeChunk(shot);
		Mix_FreeChunk(explode);
		Mix_CloseAudio();
		TTF_CloseFont(font);
		TTF_CloseFont(font2);
		PHYSFS_deinit();
		TTF_Quit();
		SDL_Quit();
		exit(0);
	}

	/*
	__ship_slot slots[20];
	slots[0].id=101;
	slots[0].w=10;
	slots[0].l=10;
	slots[0].d=10;
	slots[0].type="weapon";
	slots[0].full=false;
	slots[0].fulltype="";
	slots[0].fpk="";
	slots[0].fn="";
	slots[0].image=0;

	ship->SetAngle(0);
	//ship->Colorize(cr[curcolor], cg[curcolor], cb[curcolor]);
	//ship->FPK_NewImages(loadfpk, curship->GetImageFN());
	ship->SetMaxSpeed(10);
	ship->SetMaxWarp(130);
	ship->SetMaxSafeWarp(130);
	ship->SetTurnDelay(10);
	ship->SetWarpTurnDelay(200);
	ship->FillSlots(slots,1);
	ship->SetShootPoint(40,0);
	ship->SetOverHeatMax(20);
	ship->AddWeaponToSlot(101,"base.fpk","gfx/object/bullet.png");
	//ship->AddWeaponToSlot(102,"base.fpk","gfx/object/bullet2.png");
	ship->AddShield("gfx/ship/shield.png");
	*/
	if (LOG) printf("Loading Game...\n");
	SDL_FillRect(screen,0,0);
	centerString(font2, SCREEN_WIDTH/2, SCREEN_HEIGHT/2, "Loading Game....", white);
	SDL_Flip(screen);
	delete map;
	map = new Map;
	map->Init();
	if (LOG) printf("Setting map to scenario\n");
	mainScen->SetMap(map);
	if (LOG) printf("Loading map file\n");
	LoadMap(map, mainScen->GetMapFn());
	delete gui2;
	if (LOG) printf("Creating opening message\n");
	std::string message=mainScen->GetOpenMessage();
	gui2=MakeCenterOKWindow(message);
	if (LOG) printf("Placing ship\n");
	ship->SetAngle(mainScen->GetStartAngle());
	if (LOG) printf("Ship:   Angle: %d",mainScen->GetStartAngle());
	ship->SetSpeed(mainScen->GetStartSpeed());
	if (LOG) printf(" Speed: %d",mainScen->GetStartSpeed());
	ship->SetWarp(false);
	ship->SetX(mainScen->GetStartX());
	if (LOG) printf(" X: %d",mainScen->GetStartX());
	ship->SetY(mainScen->GetStartY());
	if (LOG) printf(" Y: %d",mainScen->GetStartY());
	ship->SetCredits(mainScen->GetStartCredits());
	if (LOG) printf(" Credits: %d\n",mainScen->GetStartCredits());
	ship->SetZoom(1);
	//if (LOG) printf("Loading images\n");
	//ship->ReloadImages();
	if (LOG) printf("Setting battle hits\n");
	ship->SetHullHitMax(20);
	ship->SetShieldHitMax(10);
	ship->ResetHits();
	if (LOG) printf("Adjusting shoot point\n");
	ship->AdjustShootPoint();
	if (LOG) printf("Adding main ship to map\n");
	map->AddMainShip(ship);
	if (LOG) printf("Starting up Scenario\n");
	mainScen->Startup();
}

static int mainLoop()
{
	if (LOG) printf("Main Loop begin\n");
	SDL_Event event;
	SDL_Surface *screen=SDL_GetVideoSurface();
	int done=0;
	char disp[100];
	int getx, gety, click;
	int ingameok=gui2->GetButtonID(150,300,SCREEN_WIDTH-300,"Go!");
	int x, y, angle, speed, starx, stary, starx2, stary2, shipx, shipy, sx, sy, savespeed, range;
	int fps, fpscount;
	int targetx=0;
	int targety=0;
	bool engage, disengage;
	angle=ship->GetAngle();
	x=ship->GetX();
	y=ship->GetY();
	double dx=ship->GetDX();
	double dy=ship->GetDY();
	int maxspeed=ship->GetMaxSpeed();
	speed=ship->GetSpeed();
	bool atwarp=ship->AtWarp();
	starx=0;
	stary=0;
	starx2=0;
	stary2=0;
	fpscount=0;
	int aiunit=-1;
	int aiunits=map->GetAICount();
	fps=0;
	range=3;
	savespeed=0;
	__ship_slot curslot=ship->GetCurWeaponSlot();
	engage=FALSE;
	disengage=FALSE;
	bool paused=FALSE;
	//setup for GUI stuff
	shipx=(SCREEN_WIDTH/2)-(ship->GetImageWidth()/2);
	shipy=(SCREEN_HEIGHT/2)-(ship->GetImageHeight()/2);
	starx=(int)(-dx/10);
	stary=(int)(-dy/10);
	starx2=(int)(-dx/20);
	stary2=(int)(-dy/20);
	sx=ship->GetSectorX();
	sy=ship->GetSectorY();
	SDL_Surface *tempimg=0;
	map->SetTarget(aiunit);
	map->CheckSectors(sx, sy);
	float zoom=1;
	if (LOG) printf("Variables set, displaying opening message\n");
	TTF_Font *font9=TTF_OpenFont("fonts/arial.ttf", 9);
	while(!done)
	{
		Slock();
		SDL_FillRect(screen,0,0);
		stars->Display(starx, stary);
		stars2->Display(starx2, stary2);
		map->DisplayOnScreenObjects(x,y,SCREEN_WIDTH,SCREEN_HEIGHT);
		ship->Display(shipx,shipy);
		map->DisplayMinimap(4,10,x,y,range);
		sprintf(disp, "%d sectors", range);
		centerString(font, 34, 80, "Sensor radius:", white);
		centerString(font, 34, 90, disp, white);
		ship->StatusDisplay(SCREEN_WIDTH-125, SCREEN_HEIGHT-195, 120, 190, font9);
		map->DisplayAITarget(5, SCREEN_HEIGHT-125,120,120,font9,targetx,targety);
		while (SDL_PollEvent(&event))
		{
			switch (event.type) {
			// close button clicked
			case SDL_QUIT:
				return -1;
				break;

			// handle the keyboard
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_RETURN:
					done = 1;
					break;
				case SDLK_ESCAPE:
					done = 1;
					break;
				default:
					break;
				}
				break;
			case SDL_MOUSEMOTION:
				SDL_GetMouseState(&getx, &gety);
				gui2->SendMouseMotion(getx,gety);
				break;
			case SDL_MOUSEBUTTONDOWN:
				SDL_GetMouseState(&getx, &gety);
				click = gui2->SendMouseClick(getx, gety);
				break;
			case SDL_MOUSEBUTTONUP:
				SDL_GetMouseState(&getx, &gety);
				if (click!=-1 && click==gui2->SendMouseClick(getx, gety))
				{
					if (click==ingameok)
					{
						gui2->Draw();
						gui2->UpdateButtons();
						Sulock();
						SDL_Flip(screen);
						done=1;
					}
				}
				gui2->UpdateButtons();
				break;
			default:
				break;
			}
		}
		gui2->Draw();
		Sulock();
		SDL_Flip(screen);
	}
	done=0;
	FPSmanager *FPS = new FPSmanager;
	SDL_initFramerate(FPS);
	SDL_setFramerate(FPS, 60);
	Uint32 time=SDL_GetTicks();
	Uint32 turntime=SDL_GetTicks();
	Uint32 checktime=SDL_GetTicks();
	char disppause[100];
	sprintf(disppause, "Press '%s' to unpause", fftinput->GetPauseStr().c_str());
	if (LOG) printf("Entering gameplay loop\n");
	while(!done)
	{
		while (SDL_PollEvent(&event))
		{
			if(event.type == SDL_QUIT) { done=-1; }
			else if(event.type == SDL_ACTIVEEVENT)
			{
				if(event.active.state==SDL_APPACTIVE || event.active.state==SDL_APPINPUTFOCUS)
				{
					if(event.active.gain==0)
					{
						paused=TRUE;
						if (LOG) printf("Game was paused because the window lost focus\n");
						if (paused)
						{
							centerString(font2, SCREEN_WIDTH/2-1, SCREEN_HEIGHT/2-8-1, "Game Paused", black);
							centerString(font2, SCREEN_WIDTH/2, SCREEN_HEIGHT/2-8, "Game Paused", yellow);
							centerString(font2, SCREEN_WIDTH/2-1, SCREEN_HEIGHT/2+8-1, disppause, black);
							centerString(font2, SCREEN_WIDTH/2, SCREEN_HEIGHT/2+8, disppause, yellow);
							SDL_Flip(screen);
						}
					}
				}
			}
			else if(event.type == SDL_KEYDOWN)
			{
				if(event.key.keysym.sym == fftinput->GetCancel())
				{
					done = InGameMenu();
				}
				else if((event.key.keysym.sym == fftinput->GetWarp1() || event.key.keysym.sym == fftinput->GetWarp2()) && !paused)
				{
					if (ship->AtWarp() && ship->WarpEnabled()) { engage=FALSE; disengage=TRUE; }
					else if (!ship->AtWarp() && ship->WarpEnabled()) { disengage=FALSE; engage=TRUE; ship->SetWarp(TRUE); }
				}
				else if((event.key.keysym.sym == fftinput->GetFire1() || event.key.keysym.sym == fftinput->GetFire2()) && !paused)
				{
					if (!ship->AtWarp())
					{
						int tempspeed=speed+maxspeed;
						if (speed<0) tempspeed=maxspeed;
						map->SpawnBullet(ship->GetShootX(), ship->GetShootY(), tempspeed, angle, curslot.fn, curslot.fpk, aiunit, curslot.id);
						Mix_PlayChannel(0,shot,0);
					}
				}
				else if(event.key.keysym.sym == fftinput->GetMiniMapZoomIn() && !paused)
				{
					if (range*2 < 20) { range=range*2; }
				}
				else if(event.key.keysym.sym == fftinput->GetMiniMapZoomOut() && !paused)
				{
					if (range/2 > 1) { range=range/2; }
				}
				else if(event.key.keysym.sym == fftinput->GetZoomIn() && !paused)
				{
					if (zoom+0.25 <= 1)
					{
						zoom+=0.25;
						if (LOG) printf("Zooming ship\n");
						ship->SetZoom(zoom);
						if (LOG) printf("Loading Ship images\n");
						ship->ReloadImages();
						if (LOG) printf("Zooming map\n");
						map->SetZoom(zoom);
					}
				}
				else if(event.key.keysym.sym == fftinput->GetZoomOut() && !paused)
				{
					if (zoom-0.25 >=0.5)
					{
						zoom-=0.25;
						if (LOG) printf("Zooming ship\n");
						ship->SetZoom(zoom);
						if (LOG) printf("Loading Ship images\n");
						ship->ReloadImages();
						if (LOG) printf("Zooming map\n");
						map->SetZoom(zoom);
					}
				}
				else if(event.key.keysym.sym == fftinput->GetPrevWeapon() && !paused)
				{
					ship->PrevWeapon();
					if (strcmp(ship->GetCurWeaponSlot().holdweapon.c_str(),"heatseeking")) aiunit=-1;
					else aiunit=0;
					map->SetTarget(aiunit);
				}
				else if(event.key.keysym.sym == fftinput->GetNextWeapon() && !paused)
				{
					ship->NextWeapon();
					if (strcmp(ship->GetCurWeaponSlot().holdweapon.c_str(),"heatseeking")) aiunit=-1;
					else aiunit=0;
					map->SetTarget(aiunit);
				}
				else if(event.key.keysym.sym == fftinput->GetDock() && !paused)
				{
					map->Dock(x,y);
					sprintf(disppause, "Press '%s' to unpause", fftinput->GetPauseStr().c_str()); //in case the user changed the pause key
				}
				else if(event.key.keysym.sym == fftinput->GetPrevTarget() && !paused && aiunit!=-1)
				{
					if (aiunit+1<aiunits) aiunit++;
					else aiunit=0;
					map->SetTarget(aiunit);
				}
				else if(event.key.keysym.sym == fftinput->GetNextTarget() && !paused && aiunit!=-1)
				{
					if (aiunit-1>=0) aiunit--;
					else aiunit=aiunits-1;
					map->SetTarget(aiunit);
				}
				else if(event.key.keysym.sym == fftinput->GetPause())
				{
					paused=!paused;
					if (paused)
					{
						centerString(font2, SCREEN_WIDTH/2-1, SCREEN_HEIGHT/2-8-1, "Game Paused", black);
						centerString(font2, SCREEN_WIDTH/2, SCREEN_HEIGHT/2-8, "Game Paused", yellow);
						centerString(font2, SCREEN_WIDTH/2-1, SCREEN_HEIGHT/2+8-1, disppause, black);
						centerString(font2, SCREEN_WIDTH/2, SCREEN_HEIGHT/2+8, disppause, yellow);
						SDL_Flip(screen);
					}
				}
			}
		}
		if (!paused)
		{
			fpscount+=1;
			bool overheated=ship->IsOverHeated();
			int hullhits=ship->GetHullHits();
			int maxhullhits=ship->GetMaxHullHits();
			shipx=(SCREEN_WIDTH/2)-(ship->GetImageWidth()/2);
			shipy=(SCREEN_HEIGHT/2)-(ship->GetImageHeight()/2);
			ship->Move(0,0);
			x=ship->GetX();
			y=ship->GetY();
			dx=ship->GetDX();
			dy=ship->GetDY();
			angle=ship->GetAngle();
			speed=ship->GetSpeed();
			sx=ship->GetSectorX();
			sy=ship->GetSectorY();
			curslot=ship->GetCurWeaponSlot();
			aiunits=map->GetAICount();
			if (strcmp(curslot.holdweapon.c_str(),"heatseeking")) aiunit=-1;
			if (aiunit<aiunits && aiunit!=-1)
			{
				targetx=map->GetAIUnitX(aiunit);
				targety=map->GetAIUnitY(aiunit);
			}
			else
			{
				targetx=0;
				targety=0;
				aiunit=-1;
				map->SetTarget(aiunit);
			}
			Slock();
			SDL_FillRect(screen,0,0);
			if (engage && speed<50)
			{
				ship->SetSpeed(speed+1);
			}
			if (engage && speed>=50)
			{
				engage=FALSE;
			}
			if (disengage && speed>savespeed)
			{
				ship->SetSpeed(speed-1);
			}
			if (disengage && speed<=savespeed)
			{
				disengage=FALSE;
				ship->SetWarp(false);
			}
			speed=ship->GetSpeed();
			map->MoveBullets();
			map->MoveAI(x,y);
			map->CheckSectors(sx, sy);
  			starx=(int)(-dx/10);
			stary=(int)(-dy/10);
			starx2=(int)(-dx/20);
			stary2=(int)(-dy/20);
			atwarp=ship->AtWarp();
			if (!atwarp)
			{
				stars->Display(starx, stary);
				stars2->Display(starx2, stary2);
				map->DisplayOnScreenObjects(x,y,SCREEN_WIDTH,SCREEN_HEIGHT);
				if (hullhits<maxhullhits && !overheated) ship->Display(shipx,shipy);
				savespeed=speed;
				if (savespeed<0) { savespeed=0; }
			}
			else if (atwarp)
			{
				stars2->DisplayWarp(starx2, stary2, angle, (int)((float)speed/2.5));
				map->DisplayOnScreenObjects(x,y,SCREEN_WIDTH,SCREEN_HEIGHT);
				if (hullhits<maxhullhits && !overheated) ship->Display(shipx,shipy);
				stars->DisplayWarp(starx, stary, angle, (int)((float)speed/2.5));
			}
			if (FPSSHOW)
			{
				sprintf(disp, "FPS: %d", fps);
				drawString(font2, SCREEN_WIDTH-80, 10, disp, white);
			}
			map->DisplayMinimap(4,10,x,y,range);
			sprintf(disp, "%d sectors", range);
			centerString(font, 34, 80, "Sensor radius:", white);
			centerString(font, 34, 90, disp, white);
			ship->StatusDisplay(SCREEN_WIDTH-125, SCREEN_HEIGHT-195, 120, 190, font9);
			map->DisplayAITarget(5, SCREEN_HEIGHT-125,120,120,font9,targetx,targety);
			if (SDL_GetTicks()>=time+1000)
			{
				fps=fpscount;
				fpscount=0;
				time=SDL_GetTicks();
			}
			Sulock();
			SDL_Flip(screen);
			keys = SDL_GetKeyState(NULL);
			// If the keys UP, DOWN, LEFT and RIGHT have been pressed
			if(keys[fftinput->GetUp()])
			{
				if (fpscount%2==0 && !atwarp && !engage && !disengage)
				{
					if (speed<ship->GetMaxSpeed())
					{
						ship->SetSpeed(speed+1);
					}
				}
				if (fpscount%2==0 && atwarp && !engage && !disengage)
				{
					if (speed<ship->GetMaxWarp())
					{
						ship->SetSpeed(speed+1);
					}
				}
			}
			if(keys[fftinput->GetDown()])
			{
				if (fpscount%2==0 && !atwarp && !engage && !disengage)
				{
					if (speed>-ship->GetMaxSpeed())
					{
						ship->SetSpeed(speed-1);
					}
				}
				if (fpscount%2==0 && atwarp && !engage && !disengage)
				{
					if (speed>50)
					{
						ship->SetSpeed(speed-1);
					}
				}
			}
			if(keys[fftinput->GetAllStop()])
			{
				if (fpscount%2==0 && !atwarp)
				{
					if (speed>0)
					{
						ship->SetSpeed(speed-1);
					}
					if (speed<0)
					{
						ship->SetSpeed(speed+1);
					}
				}
			}
			if(keys[fftinput->GetLeft()])
			{
				if (!atwarp && SDL_GetTicks()>=turntime+ship->GetTurnDelay())
				{
					ship->Rotate(0);
					turntime=SDL_GetTicks();
				}
				else
				{
					if (SDL_GetTicks()>=turntime+ship->GetWarpTurnDelay())
					{
						ship->Rotate(0);
						turntime=SDL_GetTicks();
					}
				}
			}
			if(keys[fftinput->GetRight()])
			{
				if (!atwarp && SDL_GetTicks()>=turntime+ship->GetTurnDelay())
				{
					ship->Rotate(1);
					turntime=SDL_GetTicks();
				}
				else {
					if (SDL_GetTicks()>=turntime+ship->GetWarpTurnDelay())
					{
						ship->Rotate(1);
						turntime=SDL_GetTicks();
					}
				}
			}
			if (SDL_GetTicks()>=checktime+2000)
			{
				//Check for victory and loss only once per 2 seconds
				checktime=SDL_GetTicks();
				if (hullhits>=maxhullhits) done=GameOverBox(SCREEN_WIDTH,SCREEN_HEIGHT);
				if (overheated) done=GameOverBox(SCREEN_WIDTH,SCREEN_HEIGHT);
				if (mainScen->CheckVictory())
				{
					std::string message="Congratulations!  You finished Scenario ";
					message+=mainScen->GetName();
					done=ExitToMenuBox(SCREEN_WIDTH,SCREEN_HEIGHT,message);
				}
				if (mainScen->CheckLoss())
				{
					std::string message="Game Over.  You failed to complete your objectives.";
					done=ExitToMenuBox(SCREEN_WIDTH,SCREEN_HEIGHT,message);
				}
			}
			SDL_framerateDelay(FPS);
		}
	}
	TTF_CloseFont(font9);
	delete FPS;
	return done;
}

